#include <iostream>
#include <fstream>
#include <iomanip>
#include <string>
#include <vector>
#include <stack>
#include <chrono>
using namespace std;
typedef long int li;
typedef long long ll;
long long int mod = 1000000007;
auto start = chrono::high_resolution_clock::now();

struct IntervalNode{
    li val_from;
    li val_to;
    li height;
    IntervalNode* left;
    IntervalNode* right;
    // IntervalNode(li x1, li x2) : val_from(x1), val_to(x2), left(NULL), right(NULL) {}
};

int max(li a, li b) {
    return (a > b)? a : b;  
}  

int height(IntervalNode* node) {
    if (node == NULL) {
        return 0;
    }
    return node->height;  
}  

int getBalance(IntervalNode *node) {
    if (node == NULL) { 
        return 0;
    }
    return height(node->left) - height(node->right);  
}

IntervalNode* newNode(li x1, li x2) {
    IntervalNode* node = new IntervalNode(); 
    node->val_from = x1;
    node->val_to = x2;
    node->height = 1;
    node->left = NULL;  
    node->right = NULL;  
    return(node);  
}

IntervalNode * minValueNode(IntervalNode* node) {
    IntervalNode* current = node;
    while (current->left != NULL) {
        current = current->left;
    } 
    return current;  
} 

IntervalNode* rightRotate(IntervalNode* y) {
    IntervalNode *x = y->left;  
    IntervalNode *T2 = x->right;  
    // Perform rotation  
    x->right = y;  
    y->left = T2;  
    // Update heights  
    y->height = max(height(y->left),height(y->right)) + 1;  
    x->height = max(height(x->left),height(x->right)) + 1;    
    // Return new root  
    return x;  
}  

IntervalNode* leftRotate(IntervalNode* x) {
    IntervalNode *y = x->right;  
    IntervalNode *T2 = y->left;    
    // Perform rotation  
    y->left = x;  
    x->right = T2;  
    // Update heights  
    x->height = max(height(x->left),height(x->right)) + 1;  
    y->height = max(height(y->left),height(y->right)) + 1;  
    // Return new root  
    return y;  
}  

class IntervalBBST{
    public:
    IntervalNode* head = NULL;
    vector<vector<li>> overlaps;
    bool flag;

    void search(IntervalNode* root, li x1, li x2){
        if (root && this->flag){
            if (root->val_from <= x1 && x2 <= root->val_to){
                this->flag = false;
            }else if (x1 <= root->val_from && root->val_from <= x2){
                this->search(root->left, x1, x2);
                this->overlaps.push_back({root->val_from, root->val_to});
                this->search(root->right, x1, x2);
            }else if (x1 <= root->val_to && root->val_to <= x2){
                this->search(root->left, x1, x2);
                this->overlaps.push_back({root->val_from, root->val_to});
                this->search(root->right, x1, x2);
            }else if (x2 < root->val_from){
                this->search(root->left, x1, x2);
            }else if (root->val_to < x1){
                this->search(root->right, x1, x2);
            }
        }
    }

    long long int insert(li x1, li x2, li rect_height){
        // cout << "Insert ( " << x1 << ", " << x2 << ", " << rect_height << " )"<< endl;
        long long int change = 0;
        this->flag = true;
        this->overlaps.clear();
        this->search(this->head, x1, x2);
        if (this->flag){
            // cout<<"overleaps size "<<overlaps.size()<<endl;
            if (!this->overlaps.empty()){
                if (x1 < this->overlaps[0][0]){
                    change += 2*(this->overlaps[0][0] - x1);
                }else{
                    x1 = this->overlaps[0][0];
                }

                if (this->overlaps.back().back() < x2){
                    change += 2*(x2 - this->overlaps.back().back());
                }else{
                    x2 = this->overlaps.back().back();
                }

                // cout <<" (1) "<<change<<endl;

                li past = this->overlaps[0][1];
                this->head = this->deleteNode(this->head,this->overlaps[0][0],this->overlaps[0][1]);
                for(int x = 1; x < overlaps.size(); x++){
                    // cout <<" -1- "<<x<<" : " <<overlaps[x][0]<<" "<<overlaps[x][1]<<" "<<past<<" "<<endl;
                    change += 2*(overlaps[x][0]-past);
                    change -= 2*rect_height;
                    past = this->overlaps[x][1];
                    this->head = this->deleteNode(this->head,this->overlaps[x][0],this->overlaps[x][1]);
                }
                // cout <<" (2) "<<change<<endl;
            }else {
                change = 2*(x2-x1) + 2*(rect_height);
            }
            // cout << " --- " <<x1<<" "<<x2<<endl;
            this->head = this->insertNode(this->head,x1,x2);
        }
        return (change);
    }

    IntervalNode* insertNode(IntervalNode* node, li x1, li x2) { 
        // cout << "InsertNode ( " << x1 << ", " << x2 << " )"<< endl;
        if (node == NULL) {
            return(newNode(x1,x2));  
        }else if (x2 < node->val_from) {
            node->left = insertNode(node->left, x1, x2);  
        }else if (node->val_to < x1) {
            node->right = insertNode(node->right, x1, x2);  
        }else {
            return node;  
        }

        node->height = 1 + max(height(node->left),height(node->right));  
        int balance = getBalance(node);  

        // Left Left Case  
        if (balance > 1 && x2 < node->left->val_from) {
            return rightRotate(node);  
        }// Right Right Case  
        if (balance < -1 && x1 > node->right->val_to) {
            return leftRotate(node);  
        }// Left Right Case  
        if (balance > 1 && x1 > node->left->val_to) {  
            node->left = leftRotate(node->left);  
            return rightRotate(node);  
        }// Right Left Case  
        if (balance < -1 && x2 < node->right->val_from){  
            node->right = rightRotate(node->right);  
            return leftRotate(node);  
        }  
        return (node);  
    } 

    IntervalNode* deleteNode(IntervalNode* root, li x1, li x2) {
        // cout << "DeleteNode ( " << x1 << ", " << x2 << " )"<< endl;
        if (root == NULL) {
            return root;
        }else if (root->val_from == x1 && root->val_to == x2 ){
            if (root->left == NULL) {  
                IntervalNode* temp = root->right;  
                root = NULL;
                return temp;
            }else if(root->right == NULL) {
                IntervalNode* temp = root->left;
                root = NULL;
                return temp;
            }else {
                IntervalNode* temp = minValueNode(root->right);  
                root->val_from = temp->val_from;
                root->val_to = temp->val_to;  
                root->right = this->deleteNode(root->right,temp->val_from,temp->val_to);  
            }
        }else if ( x2 < root->val_from ) {
            root->left = this->deleteNode(root->left, x1, x2);  
        }else if( root->val_to < x1) {
            root->right = this->deleteNode(root->right, x1, x2);  
        }

        if (root == NULL) {
            return root;
        }

        root->height = 1 + max(height(root->left), height(root->right));  
        int balance = getBalance(root);  

        // Left Left Case  
        if (balance > 1 && getBalance(root->left) >= 0) {  
            return rightRotate(root);  
        }// Left Right Case  
        if (balance > 1 && getBalance(root->left) < 0) {
            root->left = leftRotate(root->left);  
            return rightRotate(root);  
        }// Right Right Case  
        if (balance < -1 && getBalance(root->right) <= 0) {
            return leftRotate(root);  
        }// Right Left Case  
        if (balance < -1 &&  getBalance(root->right) > 0) {
            root->right = rightRotate(root->right);  
            return leftRotate(root);  
        }
        return (root);
    }

    void printInOrder(){
        this->inOrder(this->head);
        cout << endl;
    }

    void inOrder(IntervalNode* root){
        if(root != NULL){
            this->inOrder(root->left);  
            cout << "(" << root->val_from << "," << root->val_to << ") ";
            this->inOrder(root->right);  
        }
    }
};

int main () {
    ifstream inFile;
    ofstream outfile;
    inFile.open("Input.txt");
    outfile.open ("Output_Cpp.txt");

    int T;
    li N, K;
    ll AL,BL,CL,DL,AW,BW,CW,DW,AH,BH,CH,DH;
    vector<li> L(10000000), W(10000000), H(10000000);
    inFile >> T;
    for(int t = 0; t < T; t++){
        inFile >> N >> K;
        for(li k = 0; k < K; k++)   inFile >> L[k];
        inFile >> AL >> BL >> CL >> DL;
        for(li k = 0; k < K; k++)   inFile >> W[k];
        inFile >> AW >> BW >> CW >> DW;
        for(li k = 0; k < K; k++)   inFile >> H[k];
        inFile >> AH >> BH >> CH >> DH;
        for(li n = K; n < N; n++){
            L[n] = ( AL*L[n-2] + BL*L[n-1] + CL )%DL + 1 ;
            W[n] = ( AW*W[n-2] + BW*W[n-1] + CW )%DW + 1 ;
            H[n] = ( AH*H[n-2] + BH*H[n-1] +  CH )%DH + 1 ;
        }

        ll suma = 0, total = 1, change;
        IntervalBBST IB;
        for(li n = 0; n < N; n++){
            change = IB.insert(L[n],L[n]+W[n],H[n]);
            suma += change%mod;
            suma = (suma%mod + mod)%mod;
            total = (total*suma)%mod;
        }
        outfile << "Case #" << t+1 << ": "<<total<<"\n";
    }
    inFile.close();
    outfile.close();
    auto finish = chrono::high_resolution_clock::now();
    chrono::duration<double> elapsed = finish - start;
    cout << "Elapsed time C++: " << fixed << setprecision(2) << elapsed.count() << " seconds \n\n";
    // cin.get();
    return 0;
}